/*
  Copyright (c) 2012, Matthias Schiffer <mschiffer@universe-factory.net>
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


#ifndef _FFD_NEIGH_H_
#define _FFD_NEIGH_H_

#include "ffd.h"


static inline ffd_neigh_t* ffd_neigh_find(const ffd_iface_t *iface, const eth_addr_t *addr) {
	ffd_neigh_t *neigh;
	for (neigh = iface->neigh_list; neigh; neigh = neigh->next) {
		if (are_eth_addrs_equal(addr, &neigh->addr))
			return neigh;
	}

	return NULL;
}

static inline void ffd_neigh_ref(ffd_neigh_t *neigh) {
	neigh->ref++;
}


ffd_neigh_t* ffd_neigh_get(ffd_iface_t *iface, const eth_addr_t *addr);
void ffd_neigh_unref(ffd_neigh_t *neigh);
void ffd_neigh_unref_list(ffd_neigh_t *neigh);

uint16_t ffd_neigh_get_rxcost(const ffd_neigh_t *neigh);
uint16_t ffd_neigh_get_txcost(const ffd_neigh_t *neigh);
uint16_t ffd_neigh_get_cost(const ffd_neigh_t *neigh);
void ffd_neigh_reset(ffd_iface_t *iface, ffd_neigh_t *neigh);

#endif /* _FFD_NEIGH_H_ */
